-----------------------------------------------------
-- author: xieruidong
-- updatetime: 2015-02-26 13:47:07 
-- Description: 检测更新内容
-----------------------------------------------------
require("lfs")
local lz  = require("zlib")

LOAD_FROM_BIN = true -- 是否需要加载加密后的模块
NEED_UPDATE = true -- 是否需要检测更新
APP_NAME = "airfight" -- 游戏的英文名字
APP_PACKAGE_ROOT = "game" -- 游戏代码最外层的路径

-- 设置游戏的扩充形式，按照宽度的比例扩充
CCEGLView:sharedOpenGLView():setDesignResolutionSize(640, 960, kResolutionFixedWidth)
local sharedApplication = CCApplication:sharedApplication()
local sharedDirector = CCDirector:sharedDirector()
local target = sharedApplication:getTargetPlatform()
local winSize = sharedDirector:getWinSize()
local centerX = winSize.width/2
local centerY = winSize.height/2


local writablePath = CCFileUtils:sharedFileUtils():getWritablePath()
if target == kTargetWindows then -- 如果是win32平台，取消以下参数
   LOAD_FROM_BIN = false
   NEED_UPDATE = false
   writablePath = string.gsub(writablePath, '\\', '/')
end

print("可写路径为writable path:"..writablePath)
-- 测试服修改为0，android正式服为5，ios正式服为4
local nPackMode = 5 -- 0是测试模式,1是评测模式,2是同步模式,3是正式模式
local UPD_FOLDER = writablePath .. "upd/" -- 更新下载后的文件目录
local sBaseVerDomain = "https://47.75.67.247/resource/"
local sBaseDomain = "https://47.75.67.247/resource/"
local sBaseDomainBack = "https://47.75.67.247/resource/"
local sFileDir = "test_res/"
local PATCH_SERVER_ADDR = sBaseDomain .. sFileDir -- 内部测试更新地址
local PATCH_SERVER_ADDR_BACK = sBaseDomainBack .. sFileDir -- 内网更新备用地址
local sBaseUploadDomain = "https://47.75.62.91/account"

local bVerChanged = true -- 是否变更version的域名
local sTempVersionDir = "version/version_"
local sVerPath = sBaseDomain .. sTempVersionDir .. nPackMode .. ".txt"
local sVerPathBack = sBaseDomainBack .. sTempVersionDir .. nPackMode .. ".txt"

if(nPackMode == 1) then -- 评测模式
    sFileDir = "zone_pingce/"
elseif (nPackMode == 2) then --同步模式
    sFileDir = "zone_tongbu/"
elseif (nPackMode == 3) then --正式模式
    sFileDir = "zone_release/"
elseif (nPackMode == 4) then -- 7955, ios生产环境
    sFileDir = "ios_check/"
    sBaseVerDomain = "https://sg.7955.com/resource/"
    sBaseDomain = "https://sg.7955.com/resource/"
    sBaseDomainBack = "https://sg.7955.com/resource/"
    sBaseUploadDomain = "https://sg.7955.com/account"
elseif (nPackMode == 5) then  -- 7955，android生产环境
    sFileDir = "android_check/"
    sBaseVerDomain = "https://sg.7955.com/resource/"
    sBaseDomain = "https://sg.7955.com/resource/"
    sBaseDomainBack = "https://sg.7955.com/resource/"
    sBaseUploadDomain = "https://sg.7955.com/account"
end

local APK_SERVER_DOWN = sBaseDomain .. "apk/" -- apk资源下载地址
local FLIST = "flist.txt" -- 所有文件的MD5内容
local FZIP = "flist.zip"  -- flist.txt加密后的文件
local VER = "version.txt" -- 游戏的版本文件
local LCHR = "update.bin" -- laucher文件的加密后文件
local GAMEDB = "game.db" -- 游戏的数据表
local GAMEZIP = "game.zip" -- 游戏数据表加密后的文件

PATCH_SERVER_ADDR = sBaseDomain ..  sFileDir
PATCH_SERVER_ADDR_BACK = sBaseDomainBack .. sFileDir

if(not bVerChanged) then
    sVerPath = PATCH_SERVER_ADDR .. VER
else
    sVerPath = sBaseVerDomain .. sFileDir .. VER
end
sVerPathBack = PATCH_SERVER_ADDR_BACK .. VER


local tWords = {} -- 中文内容
tWords[1] = "资源版本:"
tWords[2] = "资源更新中..."
tWords[3] = "下载失败..."
tWords[4] = "游戏版本过低,是否重新下载游戏包?"
tWords[5] = "游戏版本过低,请到平台重新下载游戏包..."
tWords[6] = "您有"
tWords[7] = "K的资源需要更新,确定更新吗?"
tWords[8] = "M的资源需要更新,确定更新吗?"
tWords[9] = "游戏加载中."
tWords[10] = "游戏加载中.."
tWords[11] = "游戏加载中..."
tWords[12] = "更新时间："
tWords[13] = "更新内容"
tWords[14] = "更新公告"

tWords[10001] = "挂机可获得大量经验和金币"
tWords[10002] = "使用经验书才能让武将升级"
tWords[10003] = "武将试炼可获得大量经验书"
tWords[10004] = "加速出征可瞬间获得大量挂机奖励"
tWords[10005] = "挑战守将是武魂的主要来源"
tWords[10006] = "武将魂石可在七星坛获取"
tWords[10007] = "挂机设置可以屏蔽弹幕"
tWords[10008] = "点击头像可以找到挂机设置"
tWords[10009] = "联盟商队是获得高级道具的途径"
tWords[10010] = "挂机设置可以设定自动卖装备"
tWords[10011] = "强化石的主要来源在竞技场"
tWords[10012] = "粮食不足挂机没有收益"
tWords[10013] = "熔炼装备有概率获得更好的装备"
tWords[10014] = "通过打造可获得高品质装备"
tWords[10015] = "帮好友浇灌神树可获得爱心值"
tWords[10016] = "浇灌神树可获得钻石哦"
tWords[10017] = "洗炼可使装备战力大幅提升"



--enum
RequestFileType = { LCHR = 0, FLIST = 1, RES = 2, VER = 3, ZIP = 4 }

--写入
function write_content_to_file(path,content,mode)
    mode = mode or "w+b"
    local file = io.open(path, mode)
    if file then
        -- print(content)
        local hr,err = file:write(content)
        if hr == nil then
            print(err)
            return false
        end
        io.close(file)
        return true
    else
        print("can't open file:"..path)
        return false
    end
end
--写文件
local function write_file(path,content)
    -- make_dir(path)
    write_content_to_file(path,content)
end



-- 开始进入游戏
function game_start()
    --检测game.db文件,如果不存在就复制过去
    if not CCFileUtils:sharedFileUtils():isFileExist(UPD_FOLDER.."data/"..GAMEDB) then
        local db = CCFileUtils:sharedFileUtils():getFileData("res/data/"..GAMEDB)
        if db then
            write_file(UPD_FOLDER.."data/"..GAMEDB,db)
        else
            print("未读取到db文件")
        end
    end
     if LOAD_FROM_BIN then
         CCLuaLoadChunksFromZIP("framework_precompiled.zip")
     end
    if(sharedDirector and sharedDirector:getRunningScene()) then
        sharedDirector:getRunningScene():removeAllChildrenWithCleanup(true)
        CCTextureCache:sharedTextureCache():removeUnusedTextures()
    end
    require(APP_PACKAGE_ROOT..".app.MyApp").new():run()
end
----------------------------------------------------------------------------------------
-- 由于framework_precompiled也可能需要更新，所以一些常用的函数也需要提取出来
---------一些常用函数----------
local function class(classname, super)
    local superType = type(super)
    local cls

    if superType ~= "function" and superType ~= "table" then
        superType = nil
        super = nil
    end

    if superType == "function" or (super and super.__ctype == 1) then
        -- inherited from native C++ Object
        cls = {}

        if superType == "table" then
            -- copy fields from super
            for k,v in pairs(super) do cls[k] = v end
            cls.__create = super.__create
            cls.super    = super
        else
            cls.__create = super
            cls.ctor = function() end
        end

        cls.__cname = classname
        cls.__ctype = 1

        function cls.new(...)
            local instance = cls.__create(...)
            -- copy fields from class to native object
            for k,v in pairs(cls) do instance[k] = v end
            instance.class = cls
            instance:ctor(...)
            return instance
        end

    else
        -- inherited from Lua Object
        if super then
            cls = {}
            setmetatable(cls, {__index = super})
            cls.super = super
        else
            cls = {ctor = function() end}
        end

        cls.__cname = classname
        cls.__ctype = 2 -- lua
        cls.__index = cls

        function cls.new(...)
            local instance = setmetatable({}, cls)
            instance.class = cls
            instance:ctor(...)
            return instance
        end
    end

    return cls
end
--获取地址
local function get_path_from_fullpath(fullpath)
    if target == kTargetWindows then
        fullpath = string.gsub(fullpath, '/', '\\')
        return string.match(fullpath, "(.+)\\[^\\]*%.%w+$") -- windows
    end

    return string.match(fullpath, "(.+)/[^/]*%.%w+$") --*nix system

end
--获取文件名
local function get_filename_from_fullpath(fullpath)
    if target == kTargetWindows then
        fullpath = string.gsub(fullpath, '/', '\\')
        return string.match(fullpath, ".+\\([^\\]*%.%w+)$") -- windows system
    end

    return string.match(fullpath, ".+/([^/]*%.%w+)$") -- *nix system    
end



--文件是否存在
local function is_file_exists(path)
    return CCFileUtils:sharedFileUtils():isFileExist(path)
end

string.split = function(s, p)
    local rt= {}
    string.gsub(s, '[^'..p..']+', function(w) table.insert(rt, w) end )
    return rt
end

--创建目录
function mkDir(path)
    if not is_file_exists(path) then
        return lfs.mkdir(path)
    end
    return true
end

--创建目录(弃用)
local function make_dir(path)
    print(path)
    if lfs.chdir(path) then
        return
    end
    local rt = string.split(path,"/")
    local tmp = ""
    for i = 1,#rt-1 do
        folder = rt[i]
        tmp = tmp..folder.."/"
       print(tmp)
        if lfs.chdir(tmp) == nil then
            lfs.mkdir(tmp)
        end
    end
end

--删除目录
local function clean_dir(path)
    for file in lfs.dir(path) do
        if file ~= "." and file ~= ".." then
            local fullpath = path..file
            -- print(fullpath)
            local attr = lfs.attributes (fullpath)
            assert (type(attr) == "table")
            if attr.mode == "directory" then
                fullpath = fullpath.."/"
                clean_dir(fullpath)
                lfs.rmdir(fullpath)
            else
                --single file,do remove
                os.remove(fullpath)
            end
        end
    end
end




--写flist文件
local function write_flist(path,flist)
    local content = "local flist ={\n"
    content = content.."\tfiles={"
    for _,v in ipairs(flist.files) do
        if v ~= nil then
            local fileDesc = "\n\t\t{ file=\""..v.file.."\", md5=\""..v.md5.."\", size="..v.size.."},"
            content = content..fileDesc
        end
    end
    content = content.."\n\t}\n}\nreturn flist"
    write_file(path, content)
end

--写version文件
local function write_version(path,version)
    local content = ""
    if(version and version.binVer) then
        content = "local ver ={\n" .. "\tcore="..version.core..",\n"
            .. "binVer=" .. version.binVer .. ",\n"
            .."version=\""..version.version.."\"\n"
    else
        content = "local ver ={\n" .. "\tcore="..version.core..",\n"
            .."version=\""..version.version.."\"\n"
    end
    content = content.."\n}\nreturn ver"
    write_file(path, content)
end

local function lua_do_string(str)
    local fn = loadstring(str)
    if(fn ~= nil) then
        local retval = fn()
        return retval
    end
    return nil
end

function handler(obj, method)
    return function(...)
        return method(obj, ...)
    end
end

-- 获取版本号的数字
function getVerNum(version)
    if not version then
        return nil
    end
    local num = 0
    for w in string.gmatch(version,"%d+") do     --迭代出所有的数字
        num = num*10 + tonumber(w)
    end
    return num
end

----------------------------------------------------------------------------------------

--确定是否存在 writablePath..upd/ 文件夹 和 data文件夹
mkDir(UPD_FOLDER)
mkDir(UPD_FOLDER.."data/")

-----------------------------------------------------------------------------
local Launcher = class("Launcher", function()
    return SRootLayer:create()
end)

function Launcher:ctor()
    self.mIndex = 0
    self.nUpdateHandler = nil
    self.bIsBack = false

    self.isOver = false
    self.view = SCocoReader:createWidgetFromFile_json("layout/scene_update.json")
    self:setContentSize(self.view:getContentSize())
    self:addView(self.view)

    -- 处理背景和中间
    self.pLayoutAll = self:findViewByName("layout_all")
    self.pLayoutTop = self:findViewByName("layout_top")
    self.pLayoutCenter = self:findViewByName("layout_center")
    self.pLayoutCheck = self:findViewByName("layout_check")
    self.pLayoutNotice = self:findViewByName("layout_notice")
    self.layoutLogo = self:findViewByName("layout_logo")

    if(self.pLayoutCenter) then
        -- self.pLayoutCenter:setContentSize(CCSizeMake(
        --     self.pLayoutCenter:getWidth(), self:getHeight()-self.pLayoutTop:getHeight()
        --     -self.pLayoutCheck:getHeight()))
        -- self.pLayoutNotice:setContentSize(CCSizeMake(self.pLayoutNotice:getWidth(), 
        --     self.pLayoutCenter:getHeight()))
        -- self.pLayoutCenter:requestLayout()
        -- self.pLayoutNotice:requestLayout()

        self.pLayoutAll:requestLayout()
        self.pImgBg = self:findViewByName("bg_update")
        self.pImgBg:setPositionY(self:getHeight()/2-self.pLayoutCheck:getHeight())

        -- local pSprite = nil
        -- local newPoint = ccp(320, 
        --     self.pLayoutCenter:getHeight()+self.pLayoutCenter:getPositionY()-105)
        -- -- 判断文件是否存在
        -- local sChannelId = self:getCurChannelId()
        -- local sPngName = "logo3.png"
        -- local isUpExist = CCFileUtils:sharedFileUtils():isFileExist(UPD_FOLDER
        --     .."ui/" .. sChannelId .. "/" .. sPngName)
        -- local isResExist = CCFileUtils:sharedFileUtils():isFileExist("res/ui/"
        --     .. sChannelId .. "/" .. sPngName)
        -- if(isUpExist or isResExist) then
        --     pSprite = CCSprite:create("ui/" .. sChannelId .. "/" .. sPngName)
        --     newPoint = ccp(320, 
        --         self.pLayoutCenter:getHeight()+self.pLayoutCenter:getPositionY()-105)
        -- else
        --     pSprite = CCSprite:create("ui/" .. sPngName)
        --     newPoint = ccp(320, 
        --         self.pLayoutCenter:getHeight()+self.pLayoutCenter:getPositionY()-80)
        -- end

        -- if getIsTargetIpad() then
        --     newPoint = ccp(320, 1030)
        -- end
        -- pSprite:setPosition(newPoint)
        -- self.layoutLogo:addChild(pSprite, 5)
    end

    -- 处理按钮
    self.btnOk = self.view:findViewByName("btn_confirm")
    self.btnOk:setVisible(false)
    self.btnOk:setClickSelector(handler(self, self.onOkCheck))
    self.btnNo = self.view:findViewByName("btn_cancel")
    self.btnNo:setVisible(false)
    self.btnNo:setClickSelector(handler(self, self.onNoCheck))
    -- 处理文字显示
    self.var = self.view:findViewByName("txt_version")
    self.date = self.view:findViewByName("txt_date")
    self.tipTxt = self.view:findViewByName("txt_update_tips")
    self.pLbTitle = self.view:findViewByName("txt_notice_title")
    self.tipsScUpdate = self.view:findViewByName("lb_tips_sc_update")

    self.pLbOk = self.view:findViewByName("lb_confirm")
    self.pLbNo = self.view:findViewByName("lb_cancel")
    self.pLbOk:setString("更 新")
    self.pLbNo:setString("取 消")

    if self.pLbOk then
        self.pLbOk:setVisible(false)
    end
    if self.pLbNo then
        self.pLbNo:setVisible(false)
    end


    self.tipsScUpdate:setString(tWords[13])
    self.pLbTitle:setString(tWords[14])

    self.loading = self.view:findViewByName("slider_update")
    -- 初始化进度为0
    if(self.loading) then
        self.loading:setIsCanTouch(false)
        self.loading:setPercent(0)
    end
    --检查更新目录的版本号
    fileName = UPD_FOLDER..VER
	print('UPD_FOLDER..VER=',UPD_FOLDER..VER)
    local updVer = nil
    if is_file_exists(fileName) then
		print('版本文件 存在 fileName = ',fileName)
        updVer = dofile(fileName)
		print('版本文件 存在 updVer = ',updVer)
    end
    if updVer and updVer.version then -- 如果upd目录中存在版本文件，直接显示最新版本文件
        self.var:setString(tWords[1] .. updVer.version)
		print('tWords[1] .. updVer.version = ',tWords[1] .. updVer.version)
        if updVer.date then
            self.date:setString(tWords[12] .. updVer.date)
			print('tWords[12] .. updVer.date = ',tWords[12] .. updVer.date)
        end
    else
        -- 使用apk包中的version的版本号
        local verStr = CCFileUtils:sharedFileUtils():getFileData("res/"..VER)
        local installVer = lua_do_string(verStr)
		print('verStr = ',verStr)
		print('installVer = ',installVer.version)
        if(installVer and installVer.version) then
            self.var:setString(tWords[1] .. installVer.version)
        end
        if installVer and installVer.date then
            self.date:setString(tWords[12] .. installVer.date)
        end
    end
    -- 如果无需检测更新，直接滑动到进度100后进入游戏
	NEED_UPDATE = true
    if not NEED_UPDATE then
		print('无需检测更新')
        self.loading:setPercentTo(1, 100, handler( self, self.onPatchFinished))
    else
        -- -- 是否需要新的检测更新文件
        -- if LOAD_FROM_BIN then -- 重新下载update.bin
        --     self:requestFile(LCHR, RequestFileType.LCHR, 0)
        -- else
        --     self:requestFile(VER, RequestFileType.VER, 0)    
        -- end
        -- 优先下载version.txt文件
        self:runAction(CCSequence:createWithTwoActions(
            CCDelayTime:create(0.2), CCCallFunc:create(
                function (  )
                    self:requestFile(VER, RequestFileType.VER, 0)
                end)))
    end
    -- 执行提示语的刷新
    self:doUpdateTips(0, true)
    -- 执行特殊显示
    self:doShowBeforeTips()
end

function Launcher:update( dt )
end

function Launcher:onOkCheck( pView )
    -- 开始刷新
    if(self.bDownFailed ~= true) then
        local sharedScheduler = sharedDirector:getScheduler()
        self.nUpdateHandler = (sharedScheduler:scheduleScriptFunc(handler(self, self.update), 0, false))

        self.tipTxt:setString(tWords[2])
        self:requestFile(self.downloadList[1].file, RequestFileType.RES, 1)
    else
        self.tipTxt:setString(tWords[2])
        -- 如果是更新文件下载失败
        if self.bFailedType == RequestFileType.LCHR then -- 更新文件
            self:requestFile(LCHR, RequestFileType.LCHR, 0)
        elseif self.bFailedType == RequestFileType.VER then -- 版本文件
            self:requestFile(VER, RequestFileType.VER, 0)
        elseif self.bFailedType == RequestFileType.FLIST then -- md5文件
            self:requestFile(FZIP, RequestFileType.FLIST, 0)
        elseif self.bFailedType == RequestFileType.ZIP then -- db文件
            -- 暂时不做任何处理
        else -- 资源文件
            if(self.mIndex == nil) then
                self.mIndex = 1
            end
            local nTempIndex = self.mIndex - 2
            if(nTempIndex < 1) then
                nTempIndex = 1
            end
            self:requestFile(self.downloadList[nTempIndex].file,
                RequestFileType.RES, nTempIndex)
        end

    end

    if(self.btnOk) then
        self.btnOk:setVisible(false)
    end
    if self.pLbOk then
        self.pLbOk:setVisible(false)
    end
    if(self.btnNo) then
        self.btnNo:setVisible(false)
    end
    if self.pLbNo then
        self.pLbNo:setVisible(false)
    end
    self.bCanShowTips = true
    self.bDownFailed = false
end

function Launcher:onNoCheck( pView )
    if target == kTargetWindows then
        print("win32平台不直接退出")
    else
        CCDirector:sharedDirector():endToLua()
    end
end

function Launcher:onEnterFrame(dt)

end
-- 下载失败
function Launcher:onDownloadFailed( nType )
    -- 不执行刷新了提示语了
    self.bCanShowTips = false
    -- 重新设置提示语
    if(self.tipTxt) then
        self.tipTxt:setString(tWords[3])
        self.tipTxt:setColor(ccc3(255,55,55))
    end
    -- 设置重新更新按钮
    if(self.btnOk and self.pLayoutCheck) then
        self.btnOk:setVisible(true)
        self.btnOk:setPositionX(
            self.pLayoutCheck:getWidth()/2)
        if self.pLbOk then
            self.pLbOk:setVisible(true)
            self.pLbOk:setPositionX(
            self.pLayoutCheck:getWidth()/2)
        end
    end
    -- 标志为下载失败
    self.bDownFailed = true
    -- 失败的类型
    self.bFailedType = nType
    -- 增加失败记录次数
    self.nNormalSerFailedTimes = self.nNormalSerFailedTimes or 0
    self.nNormalSerFailedTimes = self.nNormalSerFailedTimes + 1
end

-- isBack 是否使用备用服务器地址
function Launcher:onRequestFinished(event,nType,index,path,isBack)
    -- 如果不是完成状态，直接返回
    if(not event or event.name~="completed") then
        -- 如果是下载失败
        if(event and event.name == "failed") then
            -- 如果是正式域名，判定是否超过失败次数
            if(not isBack) then
                self.nNormalSerFailedTimes = self.nNormalSerFailedTimes or 0
                self.nNormalSerFailedTimes = self.nNormalSerFailedTimes + 1
                if(self.nNormalSerFailedTimes >= 3) then
                    -- 如果正常域名失败3次，启用备用域名
                    PATCH_SERVER_ADDR = PATCH_SERVER_ADDR_BACK
                    -- version文件也修改
                    sVerPath = sVerPathBack
                end
            end
            self.nFailedTimes = self.nFailedTimes or 0 -- 失败的次数
            if(not isBack) then
                self.bIsBack = true
                -- 重置失败次数
                self.nFailedTimes = 0
                self:requestBackFile(path,nType,index)
            else
                if(self.nFailedTimes and self.nFailedTimes >= 2) then
                    self.bIsBack = false
                    -- 展示下载失败的界面
                    self.nFailedTimes = 0
                    self:onDownloadFailed(nType)
                else
                    self.bIsBack = true
                    -- 重置失败次数
                    self.nFailedTimes = self.nFailedTimes + 1
                    self:requestBackFile(path,nType,index)
                end
            end
        end
        return
    end
    local ok = (event.name=="completed")
    local request = event.request
    if not ok then
        --更新失败
        if isBack then
            print("error"..request:getErrorCode(),request:getErrorMessage())
            self.tipTxt:setString(tWords[3])
            self.tipTxt:setColor(ccc3(255,55,55))
            local gotoGame = CCSequence:createWithTwoActions(
               CCDelayTime:create(2.5),
               CCCallFunc:create(handler( self, self.onErrorEnd))
               )
            self:runAction(gotoGame)
        else
            self.bIsBack = true
            self:requestBackFile(path,nType,index)
        end
        return
    end

    local code = request:getResponseStatusCode()
    if code ~= 200 then
        self.tipTxt:setString(tWords[3] .. code)
        -- 展示下载失败的界面
        self:onDownloadFailed(nType)
        return
    end
    -- 重置失败次数，这样每次才能有3次重试的机会
    self.nFailedTimes = 0
    -- 取消备用域名的使用
    self.bIsBack = false
    local response = request:getResponseString()
    -- 获取返回的数据
    local data = request:getResponseData()
    if nType == RequestFileType.LCHR then
        self:onLauncherPacakgeFinished(data)
    elseif nType == RequestFileType.VER then
        self:onFileVerDownloaded(data)
    elseif nType == RequestFileType.FLIST then
        self:onFileListDownloaded(data)
    elseif nType == RequestFileType.ZIP then
        self:onZipFinished(data)
    else
        self:onSingleFileDownloaded(index,data)
    end
end

function Launcher:onZipFinished( data )
    print("db压缩包下载完成")
    local dbZip = lz.inflate()(data, "finish")
    write_file(UPD_FOLDER.."data/"..GAMEDB, dbZip)
    print("db解压写入完成")
end

function Launcher:onLauncherPacakgeFinished(data)
    print("update.bin检测更新")
    local localmd5 = nil
    if is_file_exists(UPD_FOLDER..LCHR) then
        localmd5 = CCCrypto:MD5File(UPD_FOLDER..LCHR)
    else
        path = CCFileUtils:sharedFileUtils():fullPathForFilename("res/"..LCHR)
        localmd5 = CCCrypto:MD5File(path)
    end
    print("本地update.bin md5:",localmd5)

    -- 读取服务器update.bin数据，保存在本地
    local rmf = UPD_FOLDER..LCHR..".tmp"
    -- 必须先删除多余的数据
    if CCFileUtils:sharedFileUtils():isFileExist(rmf) then
        print("存在")
        os.remove(rmf)
    else
        print("不存在")
    end
    write_file(rmf,data)
    local rmtmd5 = CCCrypto:MD5File(rmf)
    os.remove(rmf)
    print("服务器update.bin md5:", rmtmd5)
    -- 最新调整
    -- 判断md5值，如果有需求，重新载入update.bin
    if rmtmd5 ~= localmd5 then
        -- 写入新的文件
        if(data ~= nil) then
            print("新的update.bin写入")
            write_file(UPD_FOLDER .. LCHR, data)
        end
    end
    local bNeed = false
    -- 修改当前版本号
    if(self.updVer and self.newVer) then
        self.newVer.binVer = self.newVer.binVer or 0
        self.updVer.binVer = self.updVer.binVer or 0
        if self.updVer.binVer < self.newVer.binVer then
            bNeed = true
            self.updVer.binVer = self.newVer.binVer
        end
    end
    -- 重置文件当前内容
    if bNeed and self.updVer and self.updVer.version then
        print("修改version文件的binVer。。。。。。。")
        write_version(UPD_FOLDER..VER, self.updVer)
    end
    -- 检测资源更新
    self:checkVersionState()
end
-- 获取渠道号
function Launcher:getCurChannelId(  )
    -- 获取渠道号
    local channelId = nil
    local sharedApplication = CCApplication:sharedApplication()
    local target = sharedApplication:getTargetPlatform()
    if target == kTargetAndroid then
        -- android, 获取mac, imei, channelId
        local className = "com/andgame/plane/sdkmgr/SDKBridge"
        local methodName = "getString"
        local sig = "(Ljava/lang/String;)Ljava/lang/String;"
        local result3, retId = CCLuaJavaBridge.callStaticMethod(className, methodName, {"channelId"}, sig);
        if result3 and retId ~= "0" then
            channelId = retId or channelId
        end
    elseif target == kTargetIphone or target == kTargetIpad then
        local ok1, ret1 = CCLuaObjcBridge.callStaticMethod("PlatformSDk", "getChannelId", nil)
        if ok1 then
            channelId = string.format("%d", ret1)
        end
    end
    -- 默认是10000
    channelId = channelId or "10000"
    return channelId
end

function Launcher:onFileVerDownloaded(data)
    print("version下载成功")

    self.newVer = lua_do_string(data)
    -- 如果load不到数据，认为下载失败
    if(self.newVer == nil) then
        self:onDownloadFailed(RequestFileType.VER)
        return
    end

    --从安装目录中读取version文件
    local verStr = CCFileUtils:sharedFileUtils():getFileData("res/"..VER)
    local installVer = lua_do_string(verStr)
    self.var:setString(tWords[1]..installVer.version)
    if installVer.date then
        self.date:setString(tWords[12] .. installVer.date)
    end

    -- 从update目录中读取version文件
    --检查更新目录
    fileName = UPD_FOLDER..VER
    self.updVer = nil
    if is_file_exists(fileName) then
        self.updVer = dofile(fileName)
    end
    --当安装包中的code版本号和upd中的不同时,应该是重新安装了apk或者出现了错误,需要删除upd目录下的文件
    --最后把安装目录的flist重新写入upd中
    if self.updVer==nil or installVer.core ~= self.updVer.core or getVerNum(self.updVer.version) < getVerNum(installVer.version) then
        clean_dir(UPD_FOLDER)
        print("版本无法对应,清除目录")
        write_version(UPD_FOLDER..VER, installVer)
        self.updVer = installVer
    end
    self.var:setString(tWords[1]..self.updVer.version)
    if self.updVer.date then
        self.date:setString(tWords[12] .. self.updVer.date)
    end
    print("更新目录版本号:"..self.updVer.version)

    --检查code版本号
    if self.newVer.core > installVer.core then
        local bCanDown = false
        local nChannelId = self:getCurChannelId()
        local nNewCorVersion = 0
        local nOldCorVersion = 0
        if(self.newVer.core_version) then
            nNewCorVersion = self.newVer.core_version[tostring(nChannelId)] or 0
        end
        if(installVer.core_version) then
            nOldCorVersion = installVer.core_version[tostring(nChannelId)] or 0
        end
        if(nNewCorVersion and nOldCorVersion) then
            if(nOldCorVersion < nNewCorVersion) then
                bCanDown = true
            end
        end
        if(bCanDown) then
            self.tipTxt:setString(tWords[4])
            if(self.btnOk) then
                self.btnOk:setVisible(true)
            end
            if self.pLbOk then
                self.pLbOk:setVisible(true)
            end
        else
            self.tipTxt:setString(tWords[5])
            if(self.btnOk) then
                self.btnOk:setVisible(false)
            end
            if self.pLbOk then
                self.pLbOk:setVisible(false)
            end
        end
        if(self.btnOk) then
            self.btnOk.bCanDown = bCanDown
            self.btnOk.nChannelId = nChannelId
            self.btnOk:setPositionX(self:getWidth()/2)
            if self.pLbOk then
                self.pLbOk:setPositionX(self:getWidth()/2)
            end
            self.btnOk:setClickSelector(function ( pView )
                local bCanDown = pView.bCanDown
                local nChannelId =  pView.nChannelId
                if(bCanDown) then
                    local sUrl = APK_SERVER_DOWN .. "guaji_" ..
                        nChannelId .. "_" .. nNewCorVersion .. ".apk"
                    local sharedApplication = CCApplication:sharedApplication()
                    local target = sharedApplication:getTargetPlatform()
                    if target == kTargetAndroid then
                        print("开始执行android的下载")
                        local className = "com/andgame/airfight/Airfight"
                        local methodName = "doDownloadNewVersion"
                        local sig = "(Ljava/lang/String;)V"
                        CCLuaJavaBridge.callStaticMethod(className, methodName,
                            {sUrl}, sig)
                    elseif target == kTargetIphone or target == kTargetIpad then
                    else
                        print("执行游戏包下载")
                    end
                else
                    print("请耐心等待游戏包的更新")
                end
            end)
        end
        -- 游戏底层代码已经升级，清空update目录下的数据
        clean_dir(UPD_FOLDER)
        write_version(UPD_FOLDER..VER, installVer)
        self.updVer = installVer
        return
    end

    mkDir(UPD_FOLDER.."data/")
    --检测game.db文件,如果不存在就复制过去
    if not is_file_exists(UPD_FOLDER.."data/"..GAMEDB) then
        local db = CCFileUtils:sharedFileUtils():getFileData("res/data/"..GAMEDB)
        if db then
            write_file(UPD_FOLDER.."data/"..GAMEDB,db)
        else
            print("未读取到db文件")
        end
    end

    -- 判断是否需要更新udpate.bin
    self.newVer.binVer = self.newVer.binVer or 0
    self.updVer.binVer = self.updVer.binVer or 0
    local nNewBinVer = self.newVer.binVer
    local nOldBinVer = self.updVer.binVer
    print("nNewBinVer=" .. nNewBinVer .. ",nOldBinVer="..nOldBinVer)
    if(nNewBinVer > nOldBinVer) then -- 有新版本
        print("下载最新的==================")
        -- 重新下载update.bin
        self:requestFile(LCHR, RequestFileType.LCHR, 0)
        return
    end
    -- 检测资源更新
    self:checkVersionState()
end
-- 检测资源版本更新
function Launcher:checkVersionState(  )
    if(self.updVer and self.newVer) then
        --取出版本号的数字
        local ver1 = getVerNum(self.updVer.version)
        local ver2 = getVerNum(self.newVer.version)
        print("本地版本号:"..ver1.."  服务器版本号:"..ver2)
        --检测是否需要更新
        if ver2 <= ver1 then
            print("已经是最新版本")
            if(self.loading) then
                self.loading:setPercent(100)
            end
            game_start()
            return
        end
    end
    --下载flist,比较需要更新的文件
    self:requestFile(FZIP, RequestFileType.FLIST, 0)
end

-- 显示更新公告
function Launcher:showUpdateNotice( sContent )
    if(sContent and string.len(sContent) > 0) then
        self.pLayoutNotice = self:findViewByName("layout_notice")
        if(self.pLayoutNotice) then
            self.pLayoutNotice:setVisible(true)
            local pS = self.pLayoutNotice:findViewByName("scroll")
            -- 新建文本内容
            local pTxtContent = SLabelView:create()
            pTxtContent:setTextColor(ccc3(247, 235, 180))
            pTxtContent:setString(sContent)
            pTxtContent:setClickable(false)
            pTxtContent:setFontSize(24)
            pTxtContent:setDimensions(CCSizeMake(
                pS:getWidth(), 0))
            pTxtContent:setAnchorPoint(CCPointMake(0, 1))
            -- 刷新控件高度
            pTxtContent:updateTexture()
            -- 加入到滑动列表中
            pS:addView(pTxtContent)
            pS:requestLayout()

            self.date:setString(tWords[12] .. self.newVer.date)
            self.var:setString(tWords[1] .. self.newVer.version)
        end
        if(self.pBeforeTipsLabel) then
            self.pBeforeTipsLabel:setVisible(false)
        end
    else
        self:hideUpdateNotice()
    end
end
-- 隐藏更新公告
function Launcher:hideUpdateNotice(  )
    self.pLayoutNotice = self:findViewByName("layout_notice")
    if(self.pLayoutNotice) then
        self.pLayoutNotice:setVisible(false)
    end
end

function Launcher:onFileListDownloaded(data)

        local dbZip = lz.inflate()(data, "finish")
        print("解压db文件成功")

        self.newList = lua_do_string(dbZip)
        -- 如果load不到数据，认为下载失败
        if(self.newList == nil) then
            self:onDownloadFailed(RequestFileType.FLIST)
            return
        end

        --从安装目录中读取flist文件
        local flistStr = CCFileUtils:sharedFileUtils():getFileData("res/"..FLIST)
        local installList = lua_do_string(flistStr)

        --检查更新目录中的flist文件
        fileName = UPD_FOLDER..FLIST
        local updlist = nil
        if is_file_exists(fileName) then
            updlist = dofile(fileName)
        end

        --当更新目录中没有找到flist文件,把安装目录中的写进去
        if updlist==nil then
            write_flist(UPD_FOLDER..FLIST, installList)
            updlist = installList
        end

        --创建资源目录
        local dirPaths = self.newList.dirs
        for i=1,#(dirPaths) do
            mkDir(UPD_FOLDER..(dirPaths[i].name))
        end

        --比较flist,得到需要更新的文件
        local filemap = {}
        for _,v in ipairs(updlist.files) do
            filemap[v.file] = {name = v.file,md5 = v.md5}
        end
        local count = 0
        for i,v in ipairs(self.newList.files) do
            --判断哪些需要更新
            local t = filemap[v.file]
            if  t== nil or t.md5 ~= v.md5 then
                --比较本地实际文件的md5值
                local md5 = CCCrypto:MD5File(UPD_FOLDER..v.file)
                if md5 ~= v.md5 then
                    self.downloadList = self.downloadList or {}
                    table.insert(self.downloadList, v)
                    count = count + tonumber(v.size,10)
                    -- print("更新文件"..v.file.."  self.downloadList num = "..#self.downloadList)
                end
            end
        end

        if self.downloadList == nil or #self.downloadList <= 0 then
            print("没有文件可以更新")
            self:onPatchFinished()
        else
            --开始更新
            self:onShowUpdate(count)
        end

end

function Launcher:onShowUpdate( _size )
    if(_size > 0) then
        self.bCanShowTips = false
    else
        self.bCanShowTips = true
    end
    local num = (math.floor(_size/1024/1024*100))/100
    if num < 0.1 then
        num = (math.floor(_size/1024*100))/100
        self.tipTxt:setString(tWords[6]..num..tWords[7])
    else
        self.tipTxt:setString(tWords[6]..num..tWords[8])
    end
    if(_size > 0) then
        if(self.btnNo) then
            self.btnNo:setVisible(false)
        end
        if self.pLbNo then
            self.pLbNo:setVisible(false)
        end
        if(self.btnOk) then
            self.btnOk:setVisible(true)
            self.btnOk:setPositionX(
                self.pLayoutCheck:getWidth()/2)
            if self.pLbOk then
                self.pLbOk:setVisible(true)
                self.pLbOk:setPositionX(self:getWidth()/2)
            end
        end
        -- 显示更新公告
        if(self.newVer) then
            self:showUpdateNotice(self.newVer.content)
        end

    end

end

function Launcher:onSingleFileDownloaded(index,data)
    --
    local info = self.downloadList[index]
    local fullpath = UPD_FOLDER..info.file
    print("下载文件为:"..info.file)
    if info.file == "data/game.zip" then
        self:onZipFinished(data)
    else
    print("fullpath:"..fullpath)
        write_file(fullpath,data)
    end
    -- 暂时取消此判断
    local ok = true
    if ok==false then
        print("error.")
        return
    end

    --刷新进度
    local num = index/#self.downloadList*100
    self.loading:setPercentTo(0.5, num, handler(self,self.checkFile))

    self.mIndex = index

    self:onUpdateActionEnd()
end
-- 更新进度，下载下一个文件
function Launcher:onUpdateActionEnd(  )

    if self.mIndex == #self.downloadList then
        self.isOver = true
        self.var:setString(tWords[1]..self.newVer.version)
        if self.newVer.date then
            self.date:setString(tWords[12] .. self.newVer.date)
        end
        return
    end
    -- 下标加1，下载下一个需要更新的文件
    self.mIndex = self.mIndex+1
    self:requestFile(self.downloadList[self.mIndex].file,
        RequestFileType.RES, self.mIndex)
end

function Launcher:onPatchFinished()
    print("更新完成")
    --更新flist文件
	print('self.newVer = ',self.newVer)
	--print('self.newVer.version = ',self.newVer.version)
    if self.newVer and self.newVer.version then
        write_version(UPD_FOLDER..VER, self.newVer)
    end
    -- 更新flist文件
    if self.newList then
        write_flist(UPD_FOLDER..FLIST, self.newList)
    end
    if(self.loading) then
        self.loading:setPercent(100)
    end
    --进入游戏
    game_start()
end


function Launcher:checkFile()
    if self.isOver then
        sharedDirector:getScheduler():unscheduleScriptEntry(self.nUpdateHandler)
        self.var:setString(tWords[1]..self.newVer.version)
        if self.newVer.date then
            self.date:setString(tWords[12] .. self.newVer.date)
        end
        local gotoGame = CCSequence:createWithTwoActions(
           CCDelayTime:create(1.0),
           CCCallFunc:create(handler( self, self.onPatchFinished))
           )
        self:runAction(gotoGame)
        return
    end
end
function Launcher:onEnter()

end
-- 读取和下载网络数据
function Launcher:requestFile(path,type,index)
    local sFinalPath = PATCH_SERVER_ADDR..path
    if(RequestFileType.VER == type) then -- 如果是版本文件
        sFinalPath = sVerPath
    end
    if self.bIsBack then
        self:requestBackFile(path,type,index)
        return
    end
    print("sFinalPath==" .. sFinalPath)
    local request = CCHTTPRequest:createWithUrl(
        function(event) self:onRequestFinished(event,type,index,path,false)end
        , sFinalPath,kCCHTTPRequestMethodGET)
    -- 设置30秒的网络超时
    request:setTimeout(30)
    -- 开始执行网络连接
    request:start()
end

function Launcher:requestBackFile(path,type,index)
     local sFinalPath = PATCH_SERVER_ADDR_BACK..path
    if(RequestFileType.VER == type) then -- 如果是版本文件
        sFinalPath = sVerPathBack
    end
    local request = CCHTTPRequest:createWithUrl(
        function(event) self:onRequestFinished(event,type,index,path,true)end
        , sFinalPath,kCCHTTPRequestMethodGET)
    request:setTimeout(30)
    request:start()
end

function Launcher:onExit()
end

function Launcher:onErrorEnd()
    --应该判断当前是否大于最小允许版本...
    if(self.loading) then
        self.loading:setPercent(100)
    end
    --进入游戏
    game_start()
end
-- 执行检测更新前的提示语
function Launcher:doShowBeforeTips(  )
    if(self.pBeforeTipsLabel == nil) then
        self.pBeforeTipsLabel = SLabelView:create()
        self.pBeforeTipsLabel:setFontSize(22)
        self.pBeforeTipsLabel:setTextColor(ccc3(255, 255, 255))
        self.pBeforeTipsLabel:setAnchorPoint(ccp(0, 0))
        self.pBeforeTipsLabel:setString(tWords[11])
        self.pBeforeTipsLabel:updateTexture()
        self.pBeforeTipsLabel:setZOrder(20)
        if(self.loading) then
            local pParView = self.loading:getViewParent()
            if(pParView) then
                pParView:addView(self.pBeforeTipsLabel)
                self.pBeforeTipsLabel:setPosition(
                    pParView:getWidth()/2 - self.pBeforeTipsLabel:getWidth()/2,
                        self.loading:getPositionY() - 10 )
            end
            local function doChangeTxtForBefore(  )
                local sCurText = self.pBeforeTipsLabel:getString()
                if(sCurText == tWords[11]) then
                    sCurText = tWords[9]
                elseif(sCurText == tWords[10]) then
                    sCurText = tWords[11]
                elseif(sCurText == tWords[9]) then
                    sCurText = tWords[10]
                else
                    sCurText = tWords[9]
                end
                self.pBeforeTipsLabel:setString(sCurText)
                self.pBeforeTipsLabel:setVisible(false)
                self.pBeforeTipsLabel:runAction(CCSequence:createWithTwoActions(
                    CCDelayTime:create(0.5),
                    CCCallFunc:create(doChangeTxtForBefore)))
            end
            doChangeTxtForBefore()
        end
    end
end
-- 执行提示语的刷新
function Launcher:doUpdateTips( fDelayTime, bIsFirst )
    fDelayTime = fDelayTime or 5
    self.nMinIndex = self.nMinIndex or 10001
    self.nMaxIndex = self.nMaxIndex or 10017
    self.nCurIndex = self.nCurIndex or (self.nMinIndex - 1)
    if(self.bCanShowTips == nil) then
        self.bCanShowTips = true
    end
    -- 默认是第一次
    if(bIsFirst == nil) then
        bIsFirst = true
    end
    -- 第一次随机一个数
    if(bIsFirst) then
        math.randomseed(tostring(os.time()):reverse():sub(1, 6))
        math.random(1, 100000)
        self.nCurIndex = math.random(self.nMinIndex, self.nMaxIndex)
    else
        self.nCurIndex = self.nCurIndex + 1
    end
    -- 判断取值是否合法
    if(self.nCurIndex < self.nMinIndex
        or self.nCurIndex > self.nMaxIndex) then
        self.nCurIndex = self.nMinIndex
    end
    if(self.bCanShowTips) then
        if(self.tipTxt) then
            self.tipTxt:setString(tWords[self.nCurIndex])
        end
    else
        print("显示内容无需改变")
    end
    self:runAction(CCSequence:createWithTwoActions(
        CCDelayTime:create(fDelayTime),
        CCCallFunc:create(function (  )
            self:doUpdateTips(5, false)
        end)))
end

-- 发送设备信息
local function sendDeviceInfo()
    local function urlencodechar(char)
        return "%" .. string.format("%02X", string.byte(char))
    end
    local function urlencode(input)
        -- convert line endings
        input = string.gsub(tostring(input), "\n", "\r\n")
        -- escape all characters but alphanumeric, '.' and '-'
        input = string.gsub(input, "([^%w%.%- ])", urlencodechar)
        -- convert spaces to "+" symbols
        return string.gsub(input, " ", "+")
    end
    local param = {}
    local sharedApplication = CCApplication:sharedApplication()
    local target = sharedApplication:getTargetPlatform()
    local nRechargeType = 0
    if target == kTargetAndroid then
        -- android, 获取mac, imei, channelId
        local className = "com/andgame/plane/sdkmgr/SDKBridge"
        local methodName = "getString"
        local sig = "(Ljava/lang/String;)Ljava/lang/String;"
        -- 获取mac值
        local mac = "mac"
        local result1, retMac = CCLuaJavaBridge.callStaticMethod(className, methodName, {"mac"}, sig);
        if result1 and retMac ~= "0" then
            mac = retMac or mac
        end
        -- 获取imei号
        local imei = "imei"
        local result2, retImei = CCLuaJavaBridge.callStaticMethod(className, methodName, {"imei"}, sig);
        if result2 and retImei ~= "0" then
            imei = retImei or imei
        end
        -- 获取渠道号
        local channelId = "10001"
        local result3, retId = CCLuaJavaBridge.callStaticMethod(className, methodName, {"channelId"}, sig);
        if result3 and retId ~= "0" then
            channelId = retId or channelId
        end
        -- 获取充值类型
        local result4, retRechType = CCLuaJavaBridge.callStaticMethod(className, methodName, {"ownrecharge"}, sig);
        if result4 and retRechType then
            nRechargeType = tonumber(retRechType)
        end
        -- 赋值
        param.mac = mac
        param.imei = imei
        param.channelId = channelId
    elseif target == kTargetIphone or target == kTargetIpad then
        -- 获取mac值
        local mac = "ios"
        -- 获取渠道号
        local channelId = "10001"
        local ok1, ret1 = CCLuaObjcBridge.callStaticMethod("PlatformSDk", "getChannelId", nil)
        if ok1 then
            channelId = string.format("%d", ret1)
            print("........Channel ID: ..........." .. channelId .. "..............................")
        end
        -- 获取imei号
        local imei = "imei"
        local ok2, ret2 = CCLuaObjcBridge.callStaticMethod("PlatformSDk", "getDeviceIMEI", nil)
        if ok2 then
            imei = ret2
            print("........IMEI: ..........." .. imei .. "..............................")
        end
        -- 获取充值类型
        local ok3, ret3 = CCLuaObjcBridge.callStaticMethod("PlatformSDk", "getOwnrecharge", nil)
        if ok3 and ret3 then
            nRechargeType = tonumber(ret3)
        end
        CCUserDefault:sharedUserDefault():setStringForKey("ios_imei", imei)
        CCUserDefault:sharedUserDefault():flush()
        -- 赋值
        param.mac = mac
        -- 直接使用imei字段，不另外使用udid字段
        param.channelId = channelId
        param.imei = imei
    end

    local paramStr = ""
    local index = 0
    for k, v in pairs(param) do
        index = index +1
        if (index ~= 1) then
            paramStr = paramStr.."&"
        end
        paramStr = paramStr..k.."="..v
    end


    if string.len(paramStr) <= 0 then
        return
    end
    local encrypt_key = "yksg!@#tmsjyx.cn"
    paramStr = CCCrypto:encryptXXTEA(paramStr, string.len(paramStr), encrypt_key, string.len(encrypt_key))
    paramStr = CCCrypto:encodeBase64(paramStr, string.len(paramStr))
    paramStr = urlencode(paramStr)
    paramStr = string.gsub(paramStr, "%%00$", "")

    local url = sBaseUploadDomain .. "/action/user/flushEquip?p=" .. paramStr
    local request = CCHTTPRequest:createWithUrl(function(event)

    end, url, kCCHTTPRequestMethodGET)
    request:setTimeout(30)
    request:start()

    print("update devices info")
end

local lchr = Launcher.new()
local lscene = CCScene:create()
local fWidth = winSize.height / 1136 * 640
lchr:setPositionX((winSize.width-fWidth)/2)
lscene:addChild(lchr)

if sharedDirector:getRunningScene() then
    sharedDirector:replaceScene(lscene)
else
    sharedDirector:runWithScene(lscene)
end

sendDeviceInfo()
